connect to tlma%

-- make sure that tlmsrv will be able to drop and create tables
grant alterin, createin, dropin on schema VIRT to user TLMSRV%

-- procedure to delete existing procedures
create procedure ADM.DROP_PROCEDURE_IF_EXISTS(in PROCEDURE_NAME char(100))
	begin
	declare STMT char(100);
	declare MSG char(20);
	
	declare continue handler for sqlstate '42704' set MSG = 'ok';
	
	set STMT = 'DROP PROCEDURE ' concat PROCEDURE_NAME;
	prepare S1 from STMT;
	execute S1;
end%

call ADM.DROP_PROCEDURE_IF_EXISTS('VIRT.RECREATE_VIRT_TABLES')%	

drop procedure ADM.DROP_PROCEDURE_IF_EXISTS%

create procedure VIRT.RECREATE_VIRT_TABLES()
	begin
	declare MSG char(20);
	declare continue handler for sqlstate '42704' set MSG = 'ok';

	call ADM.RUN_STMT('drop table virt.cluster');
	call ADM.RUN_STMT('drop table virt.decomissioning');
	call ADM.RUN_STMT('drop table virt.guest');
	call ADM.RUN_STMT('drop table virt.guest_hist');
	call ADM.RUN_STMT('drop table virt.host');
	call ADM.RUN_STMT('drop table virt.host_hist');
	call ADM.RUN_STMT('drop table virt.root');


-------------------------------------------------------------------------------
-- Create stagging tables
-------------------------------------------------------------------------------

	call ADM.RUN_STMT('create table VIRT.ROOT
	(
		ID					bigint				not null,
		EXT_ID				varchar(100)		,
		TYPE				varchar(100)		not null,
		VM_MANAGER_ID		bigint				not null,
		SCAN_DATE		timestamp,
		PROCESSED		smallint		DEFAULT 0 not null
	)');

	call ADM.RUN_STMT('alter table VIRT.ROOT add constraint ROOT_PK primary key
	(
		ID
	)'); 

-------------------------------------------------------------------------------

	call ADM.RUN_STMT('create table VIRT.CLUSTER
	(
			ID                  bigint              not null,
			EXT_ID       		varchar(255)        ,
		NAME				varchar(255)		,
			TYPE                varchar(100)	    not null,
			PARENT_ID			bigint				not null
	)');

	call ADM.RUN_STMT('alter table VIRT.CLUSTER add constraint CLUSTER_PK primary key
	(
			ID
	)');

-------------------------------------------------------------------------------

	call ADM.RUN_STMT('create table VIRT.HOST
	(
			ID                  bigint              not null,
			EXT_ID				varchar(100)		,
			TYPE				varchar(100)		not null,
			PARENT_ID          	bigint              not null,
			PARENT_TYPE        	integer		        not null,
			HOSTNAME			varchar(100)		,
			HARDWARE_VENDOR		varchar(100)		,
			HARDWARE_MODEL		varchar(100)		,
			PROC_VENDOR			varchar(100)		,
			PROC_DESCRIPTION	varchar(500)		,
			CPU_NUMBER			integer				,
			TOTAL_CORES 		integer				,
			CONNECTION_STATE	integer				,
		POWER_STATE			integer    
	)');

	call ADM.RUN_STMT('alter table VIRT.HOST add constraint HOST_PK primary key
	(
			ID
	)');

-------------------------------------------------------------------------------

	call ADM.RUN_STMT('create table VIRT.HOST_HIST
	(
			ID                  bigint              not null,
			EXT_ID				varchar(100)		,
			TYPE				varchar(100)		not null,
			PARENT_ID          	bigint              not null,
			PARENT_TYPE        	integer		        not null,
			HOSTNAME			varchar(100)		,
			HARDWARE_VENDOR		varchar(100)		,
			HARDWARE_MODEL		varchar(100)		,
			PROC_VENDOR			varchar(100)		,
			PROC_DESCRIPTION	varchar(500)		,
			CPU_NUMBER			integer				,
			TOTAL_CORES 		integer				,
			CONNECTION_STATE	integer				,
		POWER_STATE			integer    
	)');

	call ADM.RUN_STMT('alter table VIRT.HOST_HIST add constraint HOSTH_PK primary key
	(
			ID
	)');

-------------------------------------------------------------------------------

	call ADM.RUN_STMT('create table VIRT.GUEST
	(
			ID                  bigint              not null,
			EXT_ID              varchar(100)	    ,
			TYPE				varchar(100)		not null,
			PARENT_ID          	bigint              not null,
			CPU_NUMBER			integer				,
			AGENT_ID			bigint				,
			STATE				integer				not null default 0
	)');

	call ADM.RUN_STMT('alter table VIRT.GUEST add constraint GUEST_PK primary key
	(
			ID
	)');

-------------------------------------------------------------------------------
	call ADM.RUN_STMT('create table VIRT.GUEST_HIST
	(
			ID                  bigint              not null,
			EXT_ID              varchar(100)	    ,
			TYPE				varchar(100)		not null,
			PARENT_ID          	bigint              not null,
			CPU_NUMBER			integer				,
			AGENT_ID			bigint				,
			STATE				integer				not null default 0
	)');

	call ADM.RUN_STMT('alter table VIRT.GUEST_HIST add constraint GUESTH_PK primary key
	(
			ID
	)');

-------------------------------------------------------------------------------
	call ADM.RUN_STMT('create table VIRT.DECOMISSIONING
	(
			ID                  bigint              not null,
			GUEST_EXT_ID        varchar(100)	    ,
			HOST_EXT_ID			varchar(100)		,
			AGENT_ID          	bigint              ,
			PARENT_ID          	bigint              not null,
			TYPE				varchar(100)		,
			DECOMISSION_TIME	timestamp			not null
							
	)');

	call ADM.RUN_STMT('alter table VIRT.DECOMISSIONING add constraint DECOMISSIONING_PK primary key
	(
			ID
	)');

-------------------------------------------------------------------------------
	call ADM.RUN_STMT('comment on table VIRT.ROOT is
			''Stagging table representing the virtualization hierarchy roots.''');
	call ADM.RUN_STMT('comment on column  VIRT.ROOT.ID is 
			''Internal identifier of the root.''');
	call ADM.RUN_STMT('comment on column VIRT.ROOT.EXT_ID is
		''Name of the root (external identifier).''');
	call ADM.RUN_STMT('comment on column VIRT.ROOT.TYPE is
		''Type of the virtual manager corresponding to given root.''');
	call ADM.RUN_STMT('comment on column VIRT.ROOT.VM_MANAGER_ID is
		''Internal identifier of parent VM manager.''');
	call ADM.RUN_STMT('comment on column VIRT.ROOT.SCAN_DATE is 
     ''The date of the data collection from vm manager.''');
  call ADM.RUN_STMT('comment on column VIRT.ROOT.PROCESSED is 
     ''Flag to indicate whether given scan results was already processed and can be removed. (0-pending, 1-already processed)''');

-------------------------------------------------------------------------------
	call ADM.RUN_STMT('comment on table VIRT.CLUSTER is
			''Stagging table representing the virtualization clusters.''');
	call ADM.RUN_STMT('comment on column  VIRT.CLUSTER.ID is 
			''Internal identifier of the cluster.''');
	call ADM.RUN_STMT('comment on column VIRT.CLUSTER.EXT_ID is
		''Name of the cluster (external identifier).''');
	call ADM.RUN_STMT('comment on column VIRT.CLUSTER.NAME is
		''Name of the cluster.''');	
	call ADM.RUN_STMT('comment on column VIRT.CLUSTER.TYPE is
		''Type of the virtual manager which manages cluster.''');
	call ADM.RUN_STMT('comment on column VIRT.CLUSTER.PARENT_ID is
		''Internal identifier of parent root.''');	
	
-------------------------------------------------------------------------------
	call ADM.RUN_STMT('comment on table VIRT.HOST is
			''Stagging table representing the hosts.''');
	call ADM.RUN_STMT('comment on column  VIRT.HOST.ID is 
			''Internal identifier of the host.''');
	call ADM.RUN_STMT('comment on column VIRT.HOST.EXT_ID is
		''External identifier of the host.''');
	call ADM.RUN_STMT('comment on column VIRT.HOST.TYPE is
		''Type of the virtual manager which manages host.''');    
	call ADM.RUN_STMT('comment on column VIRT.HOST.PARENT_ID is
		''Internal identifier of parent.''');
	call ADM.RUN_STMT('comment on column VIRT.HOST.PARENT_TYPE is
		''Type of the parent (either ROOT - 0 or CLUSTER - 1).''');
	call ADM.RUN_STMT('comment on column VIRT.HOST.HOSTNAME is
		''Hostname of the given host''');
	call ADM.RUN_STMT('comment on column VIRT.HOST.HARDWARE_VENDOR is
		''Host hardware vendor.''');
	call ADM.RUN_STMT('comment on column VIRT.HOST.HARDWARE_MODEL is
		''Host hardware model.''');
	call ADM.RUN_STMT('comment on column VIRT.HOST.PROC_VENDOR is
		''Vendor of the processor.''');
	call ADM.RUN_STMT('comment on column VIRT.HOST.PROC_DESCRIPTION is
		''Description string from the BIOS.''');	
	call ADM.RUN_STMT('comment on column VIRT.HOST.CPU_NUMBER is
		''Number of processors on this host.''');			
	call ADM.RUN_STMT('comment on column VIRT.HOST.TOTAL_CORES is
		''Total number of cores.''');
	call ADM.RUN_STMT('comment on column VIRT.HOST.CONNECTION_STATE is
		''State of the host (CONNECTED - 0, DISCONNECTED - 1, NOT_RESPONDING - 2, RHEV-M STATES - from 3 to 16 (UP - 3), NULL_OR_EMPTY_STATE_RECEIVED - 9998, UNDEFINED_STATE_RECEIVED - 9999).''');
	call ADM.RUN_STMT('comment on column VIRT.HOST.POWER_STATE is
		''State of the host (ON - 0, OFF - 1, UNKNOWN - 2).''');
	
-------------------------------------------------------------------------------
	call ADM.RUN_STMT('comment on table VIRT.HOST_HIST is
			''Table to store the data from VIRT.HOST table removed during pre-processing VM Manager step.''');
	call ADM.RUN_STMT('comment on column  VIRT.HOST_HIST.ID is 
			''Internal identifier of the host.''');
	call ADM.RUN_STMT('comment on column VIRT.HOST_HIST.EXT_ID is
		''External identifier of the host.''');
	call ADM.RUN_STMT('comment on column VIRT.HOST_HIST.TYPE is
		''Type of the virtual manager which manages host.''');    
	call ADM.RUN_STMT('comment on column VIRT.HOST_HIST.PARENT_ID is
		''Internal identifier of parent.''');
	call ADM.RUN_STMT('comment on column VIRT.HOST_HIST.PARENT_TYPE is
		''Type of the parent (either ROOT - 0 or CLUSTER - 1).''');
	call ADM.RUN_STMT('comment on column VIRT.HOST_HIST.HOSTNAME is
		''Hostname of the given host''');
	call ADM.RUN_STMT('comment on column VIRT.HOST_HIST.HARDWARE_VENDOR is
		''Host hardware vendor.''');
	call ADM.RUN_STMT('comment on column VIRT.HOST_HIST.HARDWARE_MODEL is
		''Host hardware model.''');
	call ADM.RUN_STMT('comment on column VIRT.HOST_HIST.PROC_VENDOR is
		''Vendor of the processor.''');
	call ADM.RUN_STMT('comment on column VIRT.HOST_HIST.PROC_DESCRIPTION is
		''Description string from the BIOS.''');	
	call ADM.RUN_STMT('comment on column VIRT.HOST_HIST.CPU_NUMBER is
		''Number of processors on this host.''');			
	call ADM.RUN_STMT('comment on column VIRT.HOST_HIST.TOTAL_CORES is
		''Total number of cores.''');
	call ADM.RUN_STMT('comment on column VIRT.HOST_HIST.CONNECTION_STATE is
		''State of the host (CONNECTED - 0, DISCONNECTED - 1, NOT_RESPONDING - 2, RHEV-M STATES - from 3 to 16 (UP - 3), NULL_OR_EMPTY_STATE_RECEIVED - 9998, UNDEFINED_STATE_RECEIVED - 9999).''');
	call ADM.RUN_STMT('comment on column VIRT.HOST_HIST.POWER_STATE is
		''State of the host (ON - 0, OFF - 1, UNKNOWN - 2).''');
	
-------------------------------------------------------------------------------
	call ADM.RUN_STMT('comment on table VIRT.GUEST is
			''Stagging table representing the guest OS.''');
	call ADM.RUN_STMT('comment on column  VIRT.GUEST.ID is 
			''Internal identifier of the guest.''');
	call ADM.RUN_STMT('comment on column VIRT.GUEST.EXT_ID is
		''External identifier of the guest.''');
	call ADM.RUN_STMT('comment on column VIRT.GUEST.TYPE is
		''Type of the virtual manager which manages guest.''');
	call ADM.RUN_STMT('comment on column VIRT.GUEST.STATE is
		''State of the guest (DEFAULT - 0, RHEV-M STATES - from 1 to 16 (UP - 1), NULL_OR_EMPTY_STATE_RECEIVED - 9998, UNDEFINED_STATE_RECEIVED - 9999).''');
	call ADM.RUN_STMT('comment on column VIRT.GUEST.PARENT_ID is
		''Internal identifier of the parent host.''');
	call ADM.RUN_STMT('comment on column VIRT.GUEST.CPU_NUMBER is
		''Number of virtual processor assigned to the guest.''');
	call ADM.RUN_STMT('comment on column VIRT.GUEST.AGENT_ID is
		''Identifier of the agent depoyed on given guest''');
-------------------------------------------------------------------------------
	call ADM.RUN_STMT('comment on table VIRT.GUEST_HIST is
			''Table to store the data from VIRT.GUEST table removed during pre-processing VM Manager step.''');
	call ADM.RUN_STMT('comment on column  VIRT.GUEST_HIST.ID is 
			''Internal identifier of the guest.''');
	call ADM.RUN_STMT('comment on column VIRT.GUEST_HIST.EXT_ID is
		''External identifier of the guest.''');
	call ADM.RUN_STMT('comment on column VIRT.GUEST_HIST.TYPE is
		''Type of the virtual manager which manages guest.''');
	call ADM.RUN_STMT('comment on column VIRT.GUEST_HIST.STATE is
		''State of the guest (DEFAULT - 0, RHEV-M STATES - from 1 to 16 (UP - 1), NULL_OR_EMPTY_STATE_RECEIVED - 9998, UNDEFINED_STATE_RECEIVED - 9999).''');
	call ADM.RUN_STMT('comment on column VIRT.GUEST_HIST.PARENT_ID is
		''Internal identifier of the parent host.''');
	call ADM.RUN_STMT('comment on column VIRT.GUEST_HIST.CPU_NUMBER is
		''Number of virtual processor assigned to the guest.''');
	call ADM.RUN_STMT('comment on column VIRT.GUEST_HIST.AGENT_ID is
		''Identifier of the agent depoyed on given guest''');
-------------------------------------------------------------------------------
	call ADM.RUN_STMT('comment on table VIRT.DECOMISSIONING is
			''Stagging table representing the decomission of partitions.''');
	call ADM.RUN_STMT('comment on column  VIRT.DECOMISSIONING.ID is 
			''Internal identifier of the decomission.''');
	call ADM.RUN_STMT('comment on column VIRT.DECOMISSIONING.GUEST_EXT_ID is
		''Name of the guest to decomission(external identifier).''');
	call ADM.RUN_STMT('comment on column VIRT.DECOMISSIONING.HOST_EXT_ID is
		''Name of the host under decomissioned guest(external identifier).''');
	call ADM.RUN_STMT('comment on column VIRT.DECOMISSIONING.TYPE is
		''Type of the virtualization technology for guest''');
	call ADM.RUN_STMT('comment on column VIRT.DECOMISSIONING.AGENT_ID is
		''Identifier of the agent to decomission.''');
	call ADM.RUN_STMT('comment on column VIRT.DECOMISSIONING.DECOMISSION_TIME is
		''Type of the virtualization technology for guest''');
	call ADM.RUN_STMT('comment on column VIRT.DECOMISSIONING.PARENT_ID is
		''Internal identifier of the parent root.''');
		
--------------------------------------------------------------------------------
	call ADM.RUN_STMT('drop trigger VIRT.ROOT_D1');
	call ADM.RUN_STMT('create trigger VIRT.ROOT_D1
	    after delete on VIRT.ROOT
	    referencing old as OLD for each row mode db2sql
	    delete from VIRT.DECOMISSIONING
	    where VIRT.DECOMISSIONING.PARENT_ID = OLD.ID');
	
	call ADM.RUN_STMT('drop trigger VIRT.ROOT_D2');
	call ADM.RUN_STMT('create trigger VIRT.ROOT_D2
	    after delete on VIRT.ROOT
	    referencing old as OLD for each row mode db2sql
	    delete from VIRT.CLUSTER
	    where VIRT.CLUSTER.PARENT_ID = OLD.ID');
	
	call ADM.RUN_STMT('drop trigger VIRT.ROOT_D3');
	call ADM.RUN_STMT('create trigger VIRT.ROOT_D3
	    after delete on VIRT.ROOT
	    referencing old as OLD for each row mode db2sql
	    delete from VIRT.HOST
	    where VIRT.HOST.PARENT_ID = OLD.ID AND VIRT.HOST.PARENT_TYPE = 0');
	
	call ADM.RUN_STMT('drop trigger VIRT.CLUSTER_D1');
	call ADM.RUN_STMT('create trigger VIRT.CLUSTER_D1
	    after delete on VIRT.CLUSTER
	    referencing old as OLD for each row mode db2sql
	    delete from VIRT.HOST
	    where VIRT.HOST.PARENT_ID = OLD.ID AND VIRT.HOST.PARENT_TYPE = 1');
	
	call ADM.RUN_STMT('drop trigger VIRT.HOST_D1');
	call ADM.RUN_STMT('create trigger VIRT.HOST_D1
	    after delete on VIRT.HOST
	    referencing old as OLD for each row mode db2sql
	    delete from VIRT.GUEST
	    where VIRT.GUEST.PARENT_ID = OLD.ID');
    
	end%
	
grant execute on procedure VIRT.RECREATE_VIRT_TABLES to user TLMSRV%

disconnect current%